C     -*- fortran -*-
C     This file is autogenerated with f2py (version:1.21.0)
C     It contains Fortran 77 wrappers to fortran functions.

      subroutine f2pywrapnucl_z (nucl_zf2pywrap, name)
      external nucl_z
      character*2 name
      double precision nucl_zf2pywrap, nucl_z
      nucl_zf2pywrap = nucl_z(name)
      end


      subroutine f2pywrapcutoff_function (cutoff_functionf2pywrap,
     & r)
      external cutoff_function
      double precision r
      double precision cutoff_functionf2pywrap, cutoff_function
      cutoff_functionf2pywrap = cutoff_function(r)
      end


      subroutine f2pywrapv0pp (v0ppf2pywrap, gamma)
      external v0pp
      double precision gamma
      double precision v0ppf2pywrap, v0pp
      v0ppf2pywrap = v0pp(gamma)
      end


      subroutine f2pywrapchg_mism (chg_mismf2pywrap, xdelta)
      external chg_mism
      double precision xdelta
      double precision chg_mismf2pywrap, chg_mism
      chg_mismf2pywrap = chg_mism(xdelta)
      end


      subroutine f2pyinitradial(setupfunc)
      external setupfunc
      double precision a
      double precision b
      double precision r(1500)
      double precision rab(1500)
      common /radial/ a,b,r,rab
      call setupfunc(a,b,r,rab)
      end

      subroutine f2pyinitrad_int(setupfunc)
      external setupfunc
      integer nr
      common /rad_int/ nr
      call setupfunc(nr)
      end

      subroutine f2pyinitorbital(setupfunc)
      external setupfunc
      double precision so(40)
      double precision zo(40)
      common /orbital/ so,zo
      call setupfunc(so,zo)
      end

      subroutine f2pyinitorb_int(setupfunc)
      external setupfunc
      integer norb
      integer ncp
      integer no(40)
      integer lo(40)
      common /orb_int/ norb,ncp,no,lo
      call setupfunc(norb,ncp,no,lo)
      end

      subroutine f2pyinitorb_char(setupfunc)
      external setupfunc
      character il(5,1)
      common /orb_char/ il
      call setupfunc(il)
      end

      subroutine f2pyinitorb_log(setupfunc)
      external setupfunc
      logical down(40)
      common /orb_log/ down
      call setupfunc(down)
      end

      subroutine f2pyinitparam(setupfunc)
      external setupfunc
      integer scheme
      integer job
      integer ifcore
      integer ncore
      double precision rsh
      double precision zel
      double precision znuc
      double precision zsh
      double precision rc(5)
      double precision rc_input(5)
      double precision cfac
      double precision rcfac
      double precision logder_radius
      integer indu(5)
      integer indd(5)
      common /param/ scheme,job,ifcore,ncore,rsh,zel,znuc,zsh,rc,r
     &c_input,cfac,rcfac,logder_radius,indu,indd
      call setupfunc(scheme,job,ifcore,ncore,rsh,zel,znuc,zsh,rc,r
     &c_input,cfac,rcfac,logder_radius,indu,indd)
      end

      subroutine f2pyinitpar_char(setupfunc)
      external setupfunc
      character*2 nameat
      character*1 ispp
      character*2 icorr
      character*3 irel
      character*4 nicore
      common /par_char/ nameat,ispp,icorr,irel,nicore
      call setupfunc(nameat,ispp,icorr,irel,nicore)
      end

      subroutine f2pyinitpar_log(setupfunc)
      external setupfunc
      logical normal
      logical polarized
      logical relativistic
      logical is_gga
      common /par_log/ normal,polarized,relativistic,is_gga
      call setupfunc(normal,polarized,relativistic,is_gga)
      end

      subroutine f2pyinitcharge(setupfunc)
      external setupfunc
      double precision cdd(1500)
      double precision cdu(1500)
      double precision cdc(1500)
      common /charge/ cdd,cdu,cdc
      call setupfunc(cdd,cdu,cdc)
      end

      subroutine f2pyinitelecpot(setupfunc)
      external setupfunc
      double precision vid(1500)
      double precision viu(1500)
      double precision vod(1500)
      double precision vou(1500)
      common /elecpot/ vid,viu,vod,vou
      call setupfunc(vid,viu,vod,vou)
      end

      subroutine f2pyinitenergy(setupfunc)
      external setupfunc
      double precision ev(40)
      double precision ep(40)
      double precision ek(40)
      double precision etot(10)
      common /energy/ ev,ep,ek,etot
      call setupfunc(ev,ep,ek,etot)
      end

      subroutine f2pyinitmixer(setupfunc)
      external setupfunc
      double precision vn1d(1500)
      double precision vn11d(1500)
      double precision vn2d(1500)
      double precision vn22d(1500)
      double precision vn1u(1500)
      double precision vn11u(1500)
      double precision vn2u(1500)
      double precision vn22u(1500)
      double precision wk1(1500)
      double precision wk2(1500)
      common /mixer/ vn1d,vn11d,vn2d,vn22d,vn1u,vn11u,vn2u,vn22u,w
     &k1,wk2
      call setupfunc(vn1d,vn11d,vn2d,vn22d,vn1u,vn11u,vn2u,vn22u,w
     &k1,wk2)
      end

      subroutine f2pyinition(setupfunc)
      external setupfunc
      double precision viod(4,1500)
      double precision viou(4,1500)
      common /ion/ viod,viou
      call setupfunc(viod,viou)
      end

      subroutine f2pyinit_BLNK_(setupfunc)
      external setupfunc
      double precision rab2(1500)
      double precision fa(1500)
      double precision fb(1500)
      common rab2,fa,fb
      call setupfunc(rab2,fa,fb)
      end

      subroutine f2pyinittm2_blk(setupfunc)
      external setupfunc
      double precision alpha
      double precision alpha1
      double precision alpha2
      double precision alpha3
      double precision alpha4
      double precision arc
      double precision brc
      double precision cdrc
      double precision delta
      double precision eigv
      double precision rc1
      double precision rc2
      double precision rc3
      double precision rc4
      double precision rc5
      double precision rc6
      double precision rc7
      double precision rc8
      double precision vap
      double precision vapp
      double precision vrc
      integer jrc
      integer lp
      double precision ar(1500)
      common /tm2_blk/ alpha,alpha1,alpha2,alpha3,alpha4,arc,brc,c
     &drc,delta,eigv,rc1,rc2,rc3,rc4,rc5,rc6,rc7,rc8,vap,vapp,vrc,jrc,lp
     &,ar
      call setupfunc(alpha,alpha1,alpha2,alpha3,alpha4,arc,brc,cdr
     &c,delta,eigv,rc1,rc2,rc3,rc4,rc5,rc6,rc7,rc8,vap,vapp,vrc,jrc,lp,a
     &r)
      end

      subroutine f2pyinitrcompat(setupfunc)
      external setupfunc
      real*8 aa_def
      real*8 bb_def
      real*8 rmax_def
      real*8 ecuts
      common /rcompat/ aa_def,bb_def,rmax_def,ecuts
      call setupfunc(aa_def,bb_def,rmax_def,ecuts)
      end

      subroutine f2pyinitlcompat(setupfunc)
      external setupfunc
      logical use_excorr
      logical use_old_cc
      logical use_new_cc
      common /lcompat/ use_excorr,use_old_cc,use_new_cc
      call setupfunc(use_excorr,use_old_cc,use_new_cc)
      end

      subroutine f2pyinitplot(setupfunc)
      external setupfunc
      double precision a(1500)
      double precision b(1500)
      double precision c(1500)
      common /plot/ a,b,c
      call setupfunc(a,b,c)
      end

      subroutine f2pyinitnonlinear(setupfunc)
      external setupfunc
      double precision arc
      double precision brc
      double precision eigv
      double precision cdrc
      double precision vrc
      double precision vap
      double precision vapp
      double precision rc1
      double precision rc2
      double precision rc3
      integer lp
      integer jrc
      common /nonlinear/ arc,brc,eigv,cdrc,vrc,vap,vapp,rc1,rc2,rc
     &3,lp,jrc
      call setupfunc(arc,brc,eigv,cdrc,vrc,vap,vapp,rc1,rc2,rc3,lp
     &,jrc)
      end

      subroutine f2pyinitcoeffs(setupfunc)
      external setupfunc
      double precision alpha
      double precision alpha1
      double precision alpha2
      double precision alpha3
      double precision alpha4
      double precision delta
      double precision gamma
      integer ang_moment
      common /coeffs/ alpha,alpha1,alpha2,alpha3,alpha4,delta,gamm
     &a,ang_moment
      call setupfunc(alpha,alpha1,alpha2,alpha3,alpha4,delta,gamma
     &,ang_moment)
      end

      subroutine f2pyinitlinear(setupfunc)
      external setupfunc
      double precision alin(5,5)
      double precision parity
      integer indx(5)
      common /linear/ alin,parity,indx
      call setupfunc(alin,parity,indx)
      end

      subroutine f2pyinitinput_s(setupfunc)
      external setupfunc
      double precision aa
      double precision bb
      double precision zion
      integer nval
      common /input_s/ aa,bb,zion,nval
      call setupfunc(aa,bb,zion,nval)
      end

      subroutine f2pyinitinput_chr(setupfunc)
      external setupfunc
      character ray(5,10)
      character title(5,10)
      common /input_chr/ ray,title
      call setupfunc(ray,title)
      end


